#!/usr/bin/env python
"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

import os, re
import sys
from resource_management.libraries.functions import format
from resource_management.libraries.script.script import Script
from resource_management.libraries.functions.version import format_stack_version
from resource_management.libraries.functions.default import default
from resource_management.libraries.functions import stack_select
from resource_management.libraries.functions import conf_select
from resource_management.libraries.functions import get_kinit_path
from resource_management.libraries.resources.hdfs_resource import HdfsResource
from resource_management.libraries.functions.get_not_managed_resources import get_not_managed_resources
from resource_management.libraries.functions import version_select_util
from resource_management.core.logger import Logger
from resource_management.core.exceptions import Fail
import status_params
import config_utils

# server configurations
config = Script.get_config()
tmp_dir = Script.get_tmp_dir()
stack_root = Script.get_stack_root()
stack_name = default("/clusterLevelParams/stack_name", None)
stack_version_buildnum = default("/commandParams/version", None)
if stack_name == "ODP":
  # Override ODP stack root
  stack_root = "/usr/odp"
  # Override ODP stack version
  stack_version_buildnum = version_select_util.get_component_version_with_stack_selector("/usr/bin/odp-select", "registry")
elif not stack_version_buildnum and stack_name:
  stack_version_buildnum = version_select_util.get_component_version_from_symlink(stack_name, "registry")

retryAble = default("/commandParams/command_retry_enabled", False)

# Version being upgraded/downgraded to
version = default("/commandParams/version", None)
registry_install_dir = os.path.join(stack_root, "current", "registry")
namenode_hosts = default("/clusterHostInfo/namenode_hosts", None)

if type(namenode_hosts) is list:
  namenode_host = namenode_hosts[0]
else:
  namenode_host = namenode_hosts

has_namenode = not namenode_host == None

stack_version_unformatted = config['clusterLevelParams']['stack_version']
stack_version_formatted = format_stack_version(stack_version_unformatted)
upgrade_direction = default("/commandParams/upgrade_direction", None)
security_enabled = config['configurations']['cluster-env']['security_enabled']
kinit_path_local = get_kinit_path(default('/configurations/kerberos-env/executable_search_paths', None))
smokeuser = config['configurations']['cluster-env']['smokeuser']
smokeuser_principal = config['configurations']['cluster-env']['smokeuser_principal_name']
smoke_user_keytab = config['configurations']['cluster-env']['smokeuser_keytab']
service_name = 'registry'
complete_service_name = 'schema-registry'
# get the correct version to use for checking stack features
#version_for_stack_feature_checks = get_stack_feature_version(config)

# When downgrading the 'version' and 'current_version' are both pointing to the downgrade-target version
# downgrade_from_version provides the source-version the downgrade is happening from
downgrade_from_version = default("/commandParams/downgrade_from_version", None)

hostname = config['agentLevelParams']['hostname']
java_home = config['ambariLevelParams']['java_home']
# default registry parameters
registry_home = os.path.join(stack_root, "current", "registry")
registry_bin = os.path.join(registry_home, "bin", "registry")

registry_managed_log_dir = os.path.join(registry_home, "logs")
conf_dir = os.path.join(registry_home, "conf")

limits_conf_dir = "/etc/security/limits.d"

registry_user_nofile_limit = default('/configurations/registry-env/registry_user_nofile_limit', 65536)
registry_user_nproc_limit = default('/configurations/registry-env/registry_user_nproc_limit', 65536)

registry_user = config['configurations']['registry-env']['registry_user']
registry_log_dir = config['configurations']['registry-env']['registry_log_dir']
registry_config_dir = config['configurations']['registry-common']['registry.config.dir']
registry_config_dir = registry_config_dir.replace('{registry_install_dir}', registry_install_dir)
registry_port = config['configurations']['registry-common']['port']
registry_admin_port = config['configurations']['registry-common']['adminPort']
registry_enable_cors = config['configurations']['registry-env']['enable_cors']
hdfs_site_config = config['configurations']['hdfs-site']
hdfs_site_attributes = config['configurationAttributes']['hdfs-site']
# This is hardcoded on the registry bash process lifecycle on which we have no control over
registry_managed_pid_dir = "/var/run/registry"

user_group = config['configurations']['cluster-env']['user_group']
java64_home = config['ambariLevelParams']['java_home']
registry_env_sh_template = config_utils.merge_env(config['configurations']['registry-env'])

if security_enabled:
  _hostname_lowercase = config['agentLevelParams']['hostname'].lower()
  registry_ui_keytab_path = config['configurations']['registry-env']['registry_ui_keytab']
  _registry_ui_jaas_principal_name = config['configurations']['registry-env']['registry_ui_principal_name']
  registry_ui_jaas_principal = _registry_ui_jaas_principal_name.replace('_HOST',_hostname_lowercase)
  registry_service_keytab_path = config['configurations']['registry-env']['registry_service_keytab']
  _registry_service_principal_name = config['configurations']['registry-env']['registry_service_principal_name']
  registry_service_principal = _registry_service_principal_name.replace('_HOST',_hostname_lowercase)
  #registry_kerberos_params = "-Djava.security.auth.login.config="+ conf_dir +"/registry_jaas.conf"
  registry_servlet_filter = config['configurations']['registry-common']['servlet.filter']
  kerberos_realm = config['configurations']['kerberos-env']['realm']
  _registry_servlet_kerberos_name_rules = config['configurations']['registry-common']['kerberos.name.rules']
  registry_servlet_kerberos_name_rules = _registry_servlet_kerberos_name_rules.replace('EXAMPLE.COM',kerberos_realm)
  ui_security_enabled = config['configurations']['registry-env']['enable_ui_security']
  #registry_servlet_token_validity = (config['configurations']['registry-common']['token.validity'])
  registry_servlet_token_validity = 36000
  if 'registry.kerberos.non-browser.user-agents' in config['configurations']['registry-common']:
    registry_kerberos_non_browser_user_agents = config['configurations']['registry-common']['registry.kerberos.non-browser.user-agents']
  else:
    registry_kerberos_non_browser_user_agents = None
else:
  ui_security_enabled = False

#SSL related configs
if 'registry-ssl-config' in config['configurations']:
  registry_ssl_enabled = config['configurations']['registry-ssl-config']['registry.ssl.isenabled']
  registry_ssl_port = config['configurations']['registry-common']['registry.ssl.port']
  registry_ssl_adminPort = config['configurations']['registry-common']['registry.ssl.adminPort']
  registry_keyStorePath = config['configurations']['registry-ssl-config']['registry.keyStorePath']
  registry_keyStorePassword = config['configurations']['registry-ssl-config']['registry.keyStorePassword']
  registry_keyStoreType = config['configurations']['registry-ssl-config']['registry.keyStoreType']
  registry_trustStorePath = config['configurations']['registry-ssl-config']['registry.trustStorePath']
  registry_trustStorePassword = config['configurations']['registry-ssl-config']['registry.trustStorePassword']
  registry_trustStoreType = config['configurations']['registry-ssl-config']['registry.trustStoreType']
  registry_validateCerts = config['configurations']['registry-ssl-config']['registry.validateCerts']
  registry_validatePeers = config['configurations']['registry-ssl-config']['registry.validatePeers']
else:
  registry_ssl_enabled = False

#SSO related configs
if 'registry-sso-config' in config['configurations']:
  registry_sso_enabled = config['configurations']['registry-sso-config']['registry.sso.enabled']
  registry_authentication_provider_url = config['configurations']['registry-sso-config']['registry.authentication.provider.url']
  registry_public_key_pem = config['configurations']['registry-sso-config']['registry.public.key.pem']
  registry_authentication_sso_token_validity = config['configurations']['registry-sso-config']['registry.token.validity']
  registry_servlet_filter = "com.hortonworks.registries.auth.server.AuthenticationFilter"

  #JWT cookie params
  registry_sso_expected_jwt_audiences = config['configurations']['registry-sso-config']['registry.expected.jwt.audiences']
  registry_sso_jwt_cookie_name = config['configurations']['registry-sso-config']['registry.jwt.cookie.name']
  registry_sso_cookie_path = config['configurations']['registry-sso-config']['registry.cookie.path']
  registry_sso_cookie_domain = config['configurations']['registry-sso-config']['registry.cookie.domain']
  registry_sso_non_brower_user_agents = config['configurations']['registry-sso-config']['registry.sso.non-browser.user-agents']
  registry_enable_trusted_proxy = config['configurations']['registry-sso-config']['registry.enable.trusted.proxy']
  if registry_enable_trusted_proxy == "true":
    registry_proxyuser_admin_hosts = config['configurations']['registry-sso-config']['registry.proxyuser.admin.hosts']
    registry_proxyuser_knox_hosts = config['configurations']['registry-sso-config']['registry.proxyuser.knox.hosts']
  else:
    registry_enable_trusted_proxy = False
else:
  registry_sso_enabled = False

# ranger host
# E.g., 2.3
stack_supports_ranger_kerberos = True
ranger_admin_hosts = default("/clusterHostInfo/ranger_admin_hosts", [])
has_ranger_admin = not len(ranger_admin_hosts) == 0
xml_configurations_supported = config['configurations']['ranger-env']['xml_configurations_supported']

ambari_server_hostname = config['clusterHostInfo']['ambari_server_host'][0]

# ranger schema registry properties
policymgr_mgr_url = config['configurations']['admin-properties']['policymgr_external_url']

if 'admin-properties' in config['configurations'] and 'policymgr_external_url' in config['configurations']['admin-properties'] and policymgr_mgr_url.endswith('/'):
  policymgr_mgr_url = policymgr_mgr_url.rstrip('/')

repo_name = str(config['clusterName']) + '_registry'

repo_config_username = config['configurations']['ranger-schema-registry-plugin-properties']['REPOSITORY_CONFIG_USERNAME']

ranger_env = config['configurations']['ranger-env']
plugin_audit_properties=config['configurations']['ranger-schema-registry-audit']
plugin_audit_attributes=config['configurationAttributes']['ranger-schema-registry-audit']
plugin_security_properties=config['configurations']['ranger-schema-registry-security']
plugin_security_attributes=config['configurationAttributes']['ranger-schema-registry-security']
plugin_policymgr_ssl_properties=config['configurations']['ranger-schema-registry-policymgr-ssl']
plugin_policymgr_ssl_attributes=config['configurationAttributes']['ranger-schema-registry-policymgr-ssl']
ranger_plugin_properties = config['configurations']['ranger-schema-registry-plugin-properties']
policy_user = config['configurations']['ranger-schema-registry-plugin-properties']['policy_user']
registry_host_name = config['agentLevelParams']['hostname']
xa_audit_hdfs_is_enabled = default('/configurations/ranger-schema-registry-audit/xasecure.audit.destination.hdfs', False)
# This enables python 2 and python 3 support. In python 3, we need to decode any byte strings
if sys.version_info[0] == 3:
  unicode = lambda text : str(text) if not isinstance(text,bytes) else text.decode('utf-8')
  import configparser as ConfigParser
else:
  import ConfigParser

if has_ranger_admin:
  enable_ranger_registry = config['configurations']['registry-env']['enable_ranger']
  repo_config_password = unicode(config['configurations']['ranger-schema-registry-plugin-properties']['REPOSITORY_CONFIG_PASSWORD'])
  previous_jdbc_jar = None
  driver_curl_source = None
  driver_curl_target = None
  ssl_keystore_password = unicode(config['configurations']['ranger-schema-registry-policymgr-ssl']['xasecure.policymgr.clientssl.keystore.password']) if xml_configurations_supported else None
  ssl_truststore_password = unicode(config['configurations']['ranger-schema-registry-policymgr-ssl']['xasecure.policymgr.clientssl.truststore.password']) if xml_configurations_supported else None
  credential_file = format('/etc/ranger/{repo_name}/cred.jceks') if xml_configurations_supported else None
  credential_file_type = 'jceks'
  ranger_admin_username = config['configurations']['ranger-env']['ranger_admin_username']
  ranger_admin_password = config['configurations']['ranger-env']['ranger_admin_password']

  #create ranger service's schema registry client properties
  registry_id_owner_for_certificate = config['configurations']['ranger-schema-registry-policymgr-ssl']['owner.for.certificate']
  regex = r"(CN)=([a-zA-Z0-9\.\-\*\[\]\|\:]*)"
  match = re.search(regex, registry_id_owner_for_certificate)
  common_name_for_certificate = match.group(2) if match else 'NONE'

  if registry_ssl_enabled:

    registry_ranger_plugin_config = {
      'schema.registry.url': format("https://{registry_host_name}:{registry_ssl_port}"),
      'keystorePath': registry_keyStorePath,
      'keystoreType': registry_keyStoreType,
      'keystorePassword': registry_keyStorePassword,
      'truststorePath': registry_trustStorePath,
      'truststoreType': registry_trustStoreType,
      'truststorePassword': registry_trustStorePassword,
      'serverCertValidation': registry_validateCerts,
      'commonNameForCertificate': common_name_for_certificate
    }
  else:
    registry_ranger_plugin_config = {
      'schema.registry.url': format("http://{registry_host_name}:{registry_port}"),
      'commonNameForCertificate': common_name_for_certificate
    }

  if stack_supports_ranger_kerberos and security_enabled:
    registry_ranger_plugin_config['policy.download.auth.users'] = registry_user
    registry_ranger_plugin_config['tag.download.auth.users'] = registry_user
    registry_ranger_plugin_config['schema-registry.authentication'] = "KERBEROS"
    ranger_registry_principal = registry_service_principal
    ranger_registry_keytab = registry_service_keytab_path
  else:
    registry_ranger_plugin_config['schema-registry.authentication'] = "NONE"

  if stack_supports_ranger_kerberos:
    registry_ranger_plugin_config['ambari.service.check.user'] = policy_user

    registry_ranger_plugin_repo = {
      'isEnabled': 'true',
      'configs': registry_ranger_plugin_config,
      'description': 'Schema registry repo',
      'name': repo_name,
      'type': 'schema-registry'
    }

  xa_audit_db_is_enabled = False
  xa_audit_db_password = None
else:
  enable_ranger_registry = False

hdfs_user = config['configurations']['hadoop-env']['hdfs_user'] if has_namenode else None
hdfs_user_keytab = config['configurations']['hadoop-env']['hdfs_user_keytab'] if has_namenode else None
hdfs_principal_name = config['configurations']['hadoop-env']['hdfs_principal_name'] if has_namenode else None
hdfs_site = config['configurations']['hdfs-site'] if has_namenode else None
default_fs = config['configurations']['core-site']['fs.defaultFS'] if has_namenode else None
hadoop_bin_dir = stack_select.get_hadoop_dir("bin") if has_namenode else None
hadoop_conf_dir = conf_select.get_hadoop_conf_dir() if has_namenode else None

local_component_list = default("/localComponents", [])
has_hdfs_client_on_node = 'HDFS_CLIENT' in local_component_list

registry_log_dir = config['configurations']['registry-env']['registry_log_dir']
registry_log_maxbackupindex = config['configurations']['registry-log4j']['registry_log_maxbackupindex']
registry_log_maxfilesize = config['configurations']['registry-log4j']['registry_log_maxfilesize']
registry_log_template = config['configurations']['registry-log4j']['content']
registry_log_template = registry_log_template.replace('{{registry_log_dir}}', registry_log_dir)
registry_log_template = registry_log_template.replace('{{registry_log_maxbackupindex}}', registry_log_maxbackupindex)
registry_log_template = registry_log_template.replace('{{registry_log_maxfilesize}}', ("%sMB" % registry_log_maxfilesize))
registry_pid_file = status_params.registry_pid_file

# flatten registry configs
jar_storage = config['configurations']['registry-common']['jar.storage']
registry_storage_database = str(config['configurations']['registry-common']['database_name'])
registry_storage_type = str(config['configurations']['registry-common']['registry.storage.type']).lower()
registry_storage_connector_connectorURI = config['configurations']['registry-common']['registry.storage.connector.connectURI']
registry_storage_connector_user = config['configurations']['registry-common']['registry.storage.connector.user']
registry_storage_connector_password = config['configurations']['registry-common']['registry.storage.connector.password']
registry_storage_query_timeout = config['configurations']['registry-common']['registry.storage.query.timeout']
registry_storage_java_class = "com.mysql.cj.jdbc.MysqlDataSource"

# database admin properties.
database_create_db_dbuser = str(config['configurations']['registry-env']['create_db_dbuser']).lower()
if database_create_db_dbuser == "true":
  database_admin_user_name = config['configurations']['registry-common']['db_root_user']
  database_admin_password = config['configurations']['registry-common']['db_root_password']
  database_admin_jdbc_url = config['configurations']['registry-common']['db_root_jdbc_url']

jar_storage_type = config['configurations']['registry-common']['jar.storage.type']
jar_storage_hdfs_url = config['configurations']['registry-common']['jar.storage.hdfs.url']
jar_storage = config['configurations']['registry-common']['jar.storage']
jar_storage_class = "com.hortonworks.registries.common.util.LocalFileSystemStorage"
jar_remote_storage_enabled  = False


if jar_storage_type != None and jar_storage_type == "hdfs":
  jar_storage_class = "com.hortonworks.registries.common.util.HdfsFileStorage"
  jar_remote_storage_enabled = True


if registry_storage_type == "postgresql":
  registry_storage_java_class = "org.postgresql.ds.PGSimpleDataSource"
elif registry_storage_type == "oracle":
  registry_storage_java_class = "oracle.jdbc.pool.OracleDataSource"
else:
  registry_storage_java_class = "com.mysql.cj.jdbc.MysqlDataSource"


#Http Proxy Configs
if 'httpProxyServer' in config['configurations']['registry-common']:
  http_proxy_server = config['configurations']['registry-common']['httpProxyServer']
else:
  http_proxy_server = None

if 'httpProxyUsername' in config['configurations']['registry-common']:
  http_proxy_username = config['configurations']['registry-common']['httpProxyUsername']
else:
  http_proxy_username = None

if 'httpProxyPassword' in config['configurations']['registry-common']:
  http_proxy_password = config['configurations']['registry-common']['httpProxyPassword']
else:
  http_proxy_password = None

# mysql jar
jdk_location = config['ambariLevelParams']['jdk_location']
if 'mysql' == registry_storage_type:
  jdbc_driver_jar = default("/ambariLevelParams/custom_mysql_jdbc_name", None)
  if jdbc_driver_jar == None:
    Logger.error("Failed to find mysql-java-connector jar. Make sure you followed the steps to register mysql driver")
    Logger.info("Users should register the mysql java driver jar.")
    Logger.info("yum install mysql-connector-java*")
    Logger.info("sudo ambari-server setup --jdbc-db=mysql --jdbc-driver=/usr/share/java/mysql-connector-java.jar")
    raise Fail('Unable to establish jdbc connection to your ' + registry_storage_type + ' instance.')

if 'oracle' == registry_storage_type:
  jdbc_driver_jar = default("/ambariLevelParams/custom_oracle_jdbc_name", None)
  if jdbc_driver_jar == None:
    Logger.error("Failed to find ojdbc jar. Please download and sure you followed the steps to register oracle driver")
    Logger.info("Users should register the oracle java driver jar.")
    Logger.info("Create a symlink e.g. ln -s /usr/share/java/ojdbc6.jar /usr/share/java/ojdbc.jar")
    Logger.info("sudo ambari-server setup --jdbc-db=oracle --jdbc-driver=/usr/share/java/ojdbc.jar")
    raise Fail('Unable to establish jdbc connection to your ' + registry_storage_type + ' instance.')

if 'postgresql' == registry_storage_type:
  jdbc_driver_jar = default("/ambariLevelParams/custom_postgres_jdbc_name", None)
  if jdbc_driver_jar == None:
    Logger.error("Failed to find postgresql jar. Please download and sure you followed the steps to register postgresql driver")
    Logger.info("Users should register the postgresql java driver jar.")
    Logger.info("Create a symlink e.g. ln -s /usr/share/java/postgresql-jdbc2.jar /usr/share/java/postgresql-jdbc.jar")
    Logger.info("Create a symlink e.g. ln -s /usr/share/java/postgresql-jdbc2ee.jar /usr/share/java/postgresql-jdbc.jar")
    Logger.info("Create a symlink e.g. ln -s /usr/share/java/postgresql-jdbc3.jar /usr/share/java/postgresql-jdbc.jar")
    Logger.info("sudo ambari-server setup --jdbc-db=postgres --jdbc-driver=/usr/share/java/postgresql-jdbc.jar")
    raise Fail('Unable to establish jdbc connection to your ' + registry_storage_type + ' instance.')

if 'mysql' == registry_storage_type or 'oracle' == registry_storage_type or 'postgresql' == registry_storage_type:
  connector_curl_source = format("{jdk_location}/{jdbc_driver_jar}")
  connector_download_dir=format("{registry_home}/libs")
  connector_bootstrap_download_dir=format("{registry_home}/bootstrap/lib")
  downloaded_custom_connector = format("{tmp_dir}/{jdbc_driver_jar}")

check_db_connection_jar_name = "DBConnectionVerification.jar"
check_db_connection_jar = format("/usr/lib/ambari-agent/{check_db_connection_jar_name}")

# bootstrap commands
jdk64_home=config['ambariLevelParams']['java_home']
bootstrap_storage_command = os.path.join(registry_home, "bootstrap", "bootstrap-storage.sh")
bootstrap_storage_run_cmd = format('export JAVA_HOME={jdk64_home} ; source {conf_dir}/registry-env.sh ; {bootstrap_storage_command}')

bootstrap_storage_intienv_command = os.path.join(registry_home, "bootstrap", "bootstrap-storage-initenv.sh")
bootstrap_storage_initevn_run_cmd = format('export JAVA_HOME={jdk64_home} ; source {conf_dir}/registry-env.sh ; {bootstrap_storage_intienv_command}')

registry_agent_dir = "/var/lib/ambari-agent/data/registry"

# oracle ssl properties
oracle_ssl_enabled = default('/configurations/registry-common/oracle_ssl_enabled', False)
oracle_ssl_version = config['configurations']['registry-common']['oracle_ssl_version']
oracle_ssl_server_dn_match = str(default('/configurations/registry-common/oracle_ssl_server_dn_match', True)).lower()
oracle_ssl_trust_store = config['configurations']['registry-common']['oracle_ssl_trust_store']
oracle_ssl_trust_store_type = config['configurations']['registry-common']['oracle_ssl_trust_store_type']
oracle_ssl_key_store = config['configurations']['registry-common']['oracle_ssl_key_store']
oracle_ssl_key_store_type = config['configurations']['registry-common']['oracle_ssl_key_store_type']

ambari_agent_conf_path = "/etc/ambari-agent/conf/ambari-agent.ini"

config = ConfigParser.ConfigParser()
config.read('/etc/ambari-agent/conf/ambari-agent.ini')
current_ambari_agent_user = config.get("agent", "run_as_user")
if (current_ambari_agent_user != "root"):
   current_ambari_agent_user = "registry"

dfs_type = default("/clusterLevelParams/dfs_type", "")
import functools
#create partial functions with common arguments for every HdfsResource call
#to create/delete hdfs directory/file/copyfromlocal we need to call params.HdfsResource in code
HdfsResource = functools.partial(
  HdfsResource,
  user=hdfs_user,
  hdfs_resource_ignore_file = "/var/lib/ambari-agent/data/.hdfs_resource_ignore",
  security_enabled = security_enabled,
  keytab = hdfs_user_keytab,
  kinit_path_local = kinit_path_local,
  hadoop_bin_dir = hadoop_bin_dir,
  hadoop_conf_dir = hadoop_conf_dir,
  principal_name = hdfs_principal_name,
  hdfs_site = hdfs_site,
  default_fs = default_fs,
  immutable_paths = get_not_managed_resources(),
  dfs_type = dfs_type
)